#ifndef _LL_ARCH_X86C_MISC_H
#define _LL_ARCH_X86C_MISC_H

#include <lowlevel/types.h>

// Call this inside busy loops to tell the CPU to save power,
// use the bus less, or other CPU-specific effects.
// This function is not privileged.

static inline void ll_busywait()
{
	asm volatile("pause" : : : "memory");
}

// The ll_ints functions are normally privileged, but on x86 can
// be also be used with IOPL 3.

static inline void ll_ints_off()
{
	asm volatile("cli" : : : "memory");
}

static inline void ll_ints_on()
{
	asm volatile("sti" : : : "memory");
}

static inline unsigned long ll_ints_save_and_off()
{
	unsigned long ret;
	asm volatile("pushf; pop %0; cli" : "=r" (ret) : : "memory");
	return ret;
}

static inline void ll_ints_restore(unsigned long saved)
{
	asm volatile("push %0; popf" : : "r" (saved) : "memory");
}

// Returns true if ints on, false if off.  This function is not privileged.

static inline int ll_get_int_state()
{
	unsigned long flags;
	asm volatile("pushf; pop %0" : "=r" (flags));
	
	return flags & 0x200;
}

// Invalidates the TLB entry for one page.  This function is privileged,
// and requires at least a 486.

static inline void ll_invalidate_tlb_entry(unsigned long addr)
{
	asm volatile("invlpg %0" : "+m" (*(char *)addr) : : "memory");
}

#endif
