// Bit manipulation functions.  These functions are not privileged.

#ifndef _LL_BITOPS_H
#define _LL_BITOPS_H

#include <lowlevel/arch.h>
#include <lowlevel/types.h>
#include _LL_INC(bitops.h)

static inline int ll_get_order_round_up(unsigned long val)
{
	return val != 1 ? ll_fls(val - 1) + 1 : 0;
}

static inline int ll_get_order_round_down(unsigned long val)
{
	return ll_fls(val);
}

// Note that the multiword bit scans are endian and word size dependent.
// They return -1 if no suitable bit was found.  Start and end are
// in bits.

static inline int ll_multiword_ffs(unsigned long *bitmap, int start, int len)
{
	static const int bits_per_long = sizeof(unsigned long) * 8;
	int off = start / bits_per_long;
	int shift_first = start % bits_per_long;

	if (shift_first) {
		unsigned long shifted = *bitmap >> shift_first;

		if (shifted)
			return ll_ffs(shifted) + start;

		off++;
		start = off * bits_per_long;
	}

	while (off < len / bits_per_long) {
		if (bitmap[off])
			break;
		
		off++;
		start += bits_per_long;
	}
	
	if (start < len && bitmap[off]) {
		int ret = start + ll_ffs(bitmap[off]);
		
		if (ret < len)
			return ret;
	}

	return -1;
}

static inline int ll_multiword_ffc(unsigned long *bitmap, int start, int len)
{
	static const int bits_per_long = sizeof(unsigned long) * 8;
	int off = start / bits_per_long;
	int shift_first = start % bits_per_long;

	if (shift_first) {
		unsigned long shifted = *bitmap >> shift_first;

		if (~shifted)
			return ll_ffs(~shifted) + start;

		off++;
		start = off * bits_per_long;
	}

	while (off < len / bits_per_long) {
		if (~bitmap[off])
			break;
		
		off++;
		start += bits_per_long;
	}
	
	if (start < len && ~bitmap[off]) {
		int ret = start + ll_ffs(~bitmap[off]);
		
		if (ret < len)
			return ret;
	}

	return -1;
}

#endif
