// arch/x64/thread.cc -- Thread switching
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following condition:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

#include <kern/thread.h>
#include <kern/mem.h>
#include <kern/pagetable.h>

namespace Arch {
	void switch_thread(Threads::Thread *dest, Threads::Thread *src)
	{
		u64 dummy1, dummy2;
		
		if (dest->addr_space) {
			assert(dest->addr_space == dest->active_addr_space);
		
			if (dest->addr_space != src->active_addr_space) {
				u64 cr3 = Mem::kvirt_to_phys(dest->addr_space->
				                             page_table->toplevel);
				asm volatile("movq %0, %%cr3" : : "r" (cr3) : "memory");
			}
		} else {
			dest->active_addr_space = src->active_addr_space;
		}
		
		Priv::tss.rsp[0] = reinterpret_cast<u64>(dest);

		asm volatile("movq %%rsp, (%0);"
		             "movq %%rbp, 8(%0);"
		             "movb $0, 16(%0);"
		             "movb 16(%1), %%al;"
		             "cmpb $0, %%al;"
		             "movq (%1), %%rsp;"
		             "movq 8(%1), %%rbp;"
		             "jnz x64_new_thread;" :
		             "=a" (dummy1), "=c" (dummy2) :
		             "0" (&src->arch.rsp), "1" (&dest->arch.rsp) :
		             "rbx", "rdx", "rsi", "rdi",
		             "r8", "r9", "r10", "r11", "r12", 
		             "r13", "r14", "r15", "memory");
	}
}
