// arch/x86/thread.cc -- Thread switching
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following condition:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

#include <kern/thread.h>
#include <kern/mem.h>
#include <kern/pagetable.h>

namespace Arch {
	void set_aspace(Mem::AddrSpace *aspace)
	{
		u32 cr3 = Mem::kvirt_to_phys(aspace->page_table->toplevel);
		asm volatile("movl %0, %%cr3" : : "r" (cr3) : "memory");
	}

	void switch_thread(Threads::Thread *dest, Threads::Thread *src)
	{
		u32 dummy1, dummy2;
		
		if (dest->addr_space) {
			assert(dest->addr_space == dest->active_addr_space);
		
			if (dest->addr_space != src->active_addr_space)
				set_aspace(dest->addr_space);
		} else {
			dest->active_addr_space = src->active_addr_space;
		}
		
		Priv::tss.esp0 = reinterpret_cast<u32>(dest);

		asm volatile("movl %%esp, (%0);"
		             "movl %%ebp, 4(%0);"
		             "movb $0, 8(%0);"
		             "movb 8(%1), %%al;"
		             "cmpb $0, %%al;"
		             "movl (%1), %%esp;"
		             "movl 4(%1), %%ebp;"
		             "jnz x86_new_thread;" :
		             "=a" (dummy1), "=c" (dummy2) :
		             "0" (&src->arch.esp), "1" (&dest->arch.esp) :
		             "ebx", "edx", "esi", "edi", "memory");
	}
}
