// core/init.cc -- C++ entry point and component initialization.
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following condition:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

#include <kern/types.h>
#include <kern/libc.h>
#include <kern/console.h>
#include <kern/arch.h>
#include <kern/time.h>
#include <kern/thread.h>

extern void *eh_frame_begin;
extern "C" void __register_frame(const void *begin);

void run_test();

extern "C" void start_kernel()
{
	run_ctors();
	Arch::arch_init();

	// __register_frame must not be called until after dynamic memory
	// allocation is initialized (exceptions wouldn't work before then,
	// anyway; if one is thrown, abort() will be called).
	
	__register_frame(&eh_frame_begin);
	printf("Starting kernel...\n");

	Time::init();
	Threads::sched.init();
	ll_ints_on();
	
	run_test();
	
	printf("done.\n");
	for(;;);
}
