#ifndef _ARCH_MEM_H
#define _ARCH_MEM_H

#include <kern/types.h>
#include <arch/paging.h>
#include <kernel/region.h>

extern int _end;

namespace Mem {
	class PageAllocZone;
};

namespace Arch {
	// First and last+1 pages of memory, as page numbers.  Small holes are
	// ignored; large holes will require discontiguous memory support.
	
	static const size_t mem_start = 0;
	extern u64 mem_end;
	
	extern uintptr_t next_free_bootmem;
	
	namespace Priv {
		// Highzone and DMA32Zone aren't currently used, but will be once high
		// memory support is added (PAE support will also be required for
		// Highzone).  KZone only goes up to 1.5GiB, to allow 512 MiB for
		// kernel virtual mappings.
		
		static const int num_zones = 2;
		
		static const System::Mem::Region mem_zone_regions[num_zones] = {
			{ 0, 16*1024*1024 - 1 }, // ISA DMA zone
			{ 16*1024*1024, 1536*1024*1024 - 1 }, // Kernel direct mappable (KZone)
#if 0
			{ 1536*1024*1024, 4ULL*1024*1024*1024 - 1 }, // DMA32Zone
			{ 4ULL*1024*1024*1024, 36ULL*1024*1024*1024 - 1} // HighZone
#endif
		};
		
		extern Mem::PageAllocZone pagezones[num_zones];
	}
	
	extern Mem::PageAllocZone **pagezonelists[Priv::num_zones];

	namespace Priv {
		struct Descriptor {
			u16 limit_low;
			u16 base_low;
			u8 base_mid;
			u8 type:4;
			u8 user:1;
			u8 dpl:2;
			u8 present:1;
			u8 limit_high:4;
			u8 sw:1;
			u8 reserved:1;
			u8 opsize:1;
			u8 gran:1;
			u8 base_high;
		};

		void early_adjust_mappings();

		// Call after mem_end is set.
		void map_physmem();
	}
}

extern Arch::Priv::Descriptor x86_gdt[1024];

#endif
