// lib/ctors.cc -- Run C++ global constructors
//
// Destructors are not run at shutdown, as we won't be relying on global
// destructors to do any cleanup.  Constuctors are needed even if we don't
// explicitly use them, however, as GCC sometimes uses them implicitly
// to initialize global data.

#include <kern/libc.h>
#include <kern/mem.h>
#include <lowlevel/barriers.h>

void run_ctors()
{
	typedef void (*ctor)();
	extern ctor ctors;
	
	for (ctor *ptr = &ctors; *ptr; ptr++)
		(*ptr)();
}

// Global constructors call this to register global destructors
// with some versions and/or configurations of GCC.  I'm not sure
// why just using a static dtor table isn't good enough.

extern "C" int __cxa_atexit(void (*func)(void *), void *arg, void *d)
{
	return 0;
}

// More crap we don't care about (we don't use global destructors),
// but GCC requires.

int __dso_handle;
