// orb/invoke.cc -- Method Invocation
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following condition:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

#include <kern/types.h>
#include <kern/orb.h>
#include <kern/pagealloc.h>
#include <kern/compiler.h>
#include <arch/usercopy.h>
#include <orb.h>

using System::RunTime::ParamInfoBlock;
using namespace ORB;
using Threads::Thread;

namespace ORB {
	static CallFrame *new_frame(Thread *thread)
	{
		CallStackHeader *hdr = thread->orbstack_top_hdr;
		
		if (unlikely(thread->orbstack_top == hdr->num_frames - 1)) {
			hdr = static_cast<CallStackHeader *>(Mem::alloc_pages(1));
			new(hdr) CallStackHeader;
			hdr->thread = thread;
			hdr->num_frames = (Arch::page_size - sizeof(CallStackHeader)) /
			                  sizeof(CallFrame);

			thread->orbstack_top_hdr->node.add_front(&hdr->node);
			thread->orbstack_top_hdr = hdr;
			thread->orbstack_top = 0;
			return &hdr->frames[0];
		}
		
		return &hdr->frames[thread->orbstack_top += 1];
	}
}

extern "C" void invoke_method(ulong objid, ulong methid,
                              ParamInfoBlock *user_pib, ulong ret_pc)
{
	ParamInfoBlock pib = Arch::copyin(user_pib);
	CallFrame *frame = new_frame(curthread);
	
	frame->object = objid;
	frame->method = methid;
	frame->caller_user_pib = user_pib;
	frame->ret_pc = ret_pc;
	
	printf("invoke_method: frame %p object %lx method %lx pib %p ret %lx\n",
	       frame, frame->object, frame->method, frame->caller_user_pib,
	       frame->ret_pc);
	
	
}
