// tests/mutex.cc
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal with
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following condition:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE.

#include <kern/types.h>
#include <kern/libc.h>
#include <kern/time.h>
#include <kern/thread.h>
#include <util/lock.h>

Lock::Lock testlock;

void threadb(void *arg)
{
	testlock.lock();
	
	while (true) {
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		while (true) {
			printf("B");
		
			Time::Time now2;
			Time::monotonic_clock.get_time(&now2);

			if (now.seconds != now2.seconds)
				break;
		}
		
		testlock.unlock();
		testlock.lock();
	}
}

void threada(void *arg)
{
	Threads::sched.new_thread(threadb, NULL, "thread b")->wake();

	testlock.lock();
	
	while (true) {
		Time::Time now;
		Time::monotonic_clock.get_time(&now);
		
		while (true) {
			printf("A");
		
			Time::Time now2;
			Time::monotonic_clock.get_time(&now2);

			if (now.seconds != now2.seconds)
				break;
		}
		
		testlock.unlock();
		testlock.lock();
	}
}


void run_test()
{
	Threads::sched.new_thread(threada, NULL, "thread a")->wake();
}
